-- Author: VertexFloat
-- Date: 09.12.2021
-- Version: Farming Simulator 22, 1.0.0.0
-- Copyright (C): VertexFloat, GIANTS Software, All Rights Reserved
-- Combine Discharge Context for Manual Combine Discharge

CombineDischargeContext = {}
local CombineDischargeContext_mt = Class(CombineDischargeContext, HUDDisplayElement)

CombineDischargeContext.CONTEXT_ICON = {
	DISCHARGE = "discharge"
}

CombineDischargeContext.MIN_DISPLAY_DURATION = 100

function CombineDischargeContext.new(hudAtlasPath, inputDisplayManager)
	local backgroundOverlay = CombineDischargeContext.createBackground()
	local self = CombineDischargeContext:superClass().new(backgroundOverlay, nil, CombineDischargeContext_mt)
	
	self.uiScale = 1
	self.inputDisplayManager = inputDisplayManager
	self.inputGlyphElement = nil
	self.contextIconElements = {}
	self.contextAction = ""
	self.contextIconName = ""
	self.targetText = ""
	self.actionText = ""
	self.contextPriority = -math.huge
	self.contextIconElementRightX = 0
	self.contextIconOffsetY = 0
	self.contextIconOffsetX = 0
	self.contextIconSizeX = 0
	self.actionTextOffsetY = 0
	self.actionTextOffsetX = 0
	self.actionTextSize = 0
	self.targetTextOffsetY = 0
	self.targetTextOffsetX = 0
	self.targetTextSize = 0
	self.borderOffsetX = 0
	self.displayTime = 0
	
	self:createComponents(hudAtlasPath, inputDisplayManager)
	
	return self
end

function CombineDischargeContext:setContext(contextAction, contextIconName, targetText, priority, actionText)
	if priority == nil then
		priority = 0
	end
	
	if self.contextPriority <= priority and self.contextIconElements[contextIconName] ~= nil then
		self.contextAction = contextAction
		self.contextIconName = contextIconName
		self.targetText = targetText
		self.contextPriority = priority
		
		local eventHelpElement = self.inputDisplayManager:getEventHelpElementForAction(self.contextAction)
		
		self.contextEventHelpElement = eventHelpElement
		
		if eventHelpElement ~= nil then
			self.inputGlyphElement:setAction(contextAction)
			self.actionText = utf8ToUpper(actionText or eventHelpElement.textRight or eventHelpElement.textLeft)
			local targetTextWidth = getTextWidth(self.targetTextSize, self.targetText)
			
			self.rightSideX = 0.5 - targetTextWidth * 0.5
			local contextIconWidth = 0
			local posX = self.rightSideX + self.contextIconOffsetX
			
			for name, element in pairs(self.contextIconElements) do
				element:setPosition(posX - element:getWidth(), nil)
				
				if name == self.contextIconName then
					contextIconWidth = element:getWidth()
				end
			end
			
			posX = posX - self.inputGlyphElement:getWidth() + self.inputIconOffsetX - contextIconWidth
			self.inputGlyphElement:setPosition(posX, nil)
		end
		
		if not self:getVisible() then
			self:setVisible(true, true)
		end
	end
	
	for name, element in pairs(self.contextIconElements) do
		element:setVisible(name == self.contextIconName)
	end
	
	self.displayTime = CombineDischargeContext.MIN_DISPLAY_DURATION
end

function CombineDischargeContext:update(dt)
	CombineDischargeContext:superClass().update(self, dt)
	
	self.displayTime = self.displayTime - dt
	local isVisible = self:getVisible()
	
	if self.displayTime <= 0 and isVisible and self.animation:getFinished() then
		self:setVisible(false, true)
	end
	
	if not self.animation:getFinished() then
		self:storeScaledValues()
	elseif self.contextAction ~= "" and not isVisible then
		self:resetContext()
	end
end

function CombineDischargeContext:resetContext()
	self.contextAction = ""
	self.contextIconName = ""
	self.targetText = ""
	self.actionText = ""
	self.contextPriority = -math.huge
end

function CombineDischargeContext:draw()
	if self.contextAction ~= "" and self.contextEventHelpElement ~= nil then
		self.inputGlyphElement:setAction(self.contextAction)
		CombineDischargeContext:superClass().draw(self)
		local _, baseY = self:getPosition()
		
		setTextColor(unpack(CombineDischargeContext.COLOR.ACTION_TEXT))
		setTextBold(true)
		setTextAlignment(RenderText.ALIGN_LEFT)
		local height = self:getHeight()
		local posX = self.rightSideX
		local posY = baseY + height * 0.5 + self.targetTextSize * 0.5 + self.actionTextOffsetY
		
		renderText(posX, posY, self.actionTextSize, self.actionText)
		posY = baseY + height * 0.5
		setTextColor(unpack(CombineDischargeContext.COLOR.TARGET_TEXT))
		setTextBold(false)
		local width = self:getWidth()
		local textWrapWidth = width - self.targetTextOffsetX - self.contextIconSizeX - self.inputGlyphElement:getWidth() * 2 - self.contextIconOffsetX
		
		setTextVerticalAlignment(RenderText.VERTICAL_ALIGN_MIDDLE)
		setTextAlignment(RenderText.ALIGN_CENTER)
		renderText(0.5, posY, self.targetTextSize, self.targetText)
		setTextVerticalAlignment(RenderText.VERTICAL_ALIGN_BASELINE)
		
		if g_uiDebugEnabled then
			local yPixel = 1 / g_screenHeight
			
			setOverlayColor(GuiElement.debugOverlay, 0, 1, 1, 1)
			renderOverlay(GuiElement.debugOverlay, posX, posY, textWrapWidth, yPixel)
		end
	end
end

function CombineDischargeContext:getWidth()
	return 1
end

function CombineDischargeContext:setScale(uiScale)
	CombineDischargeContext:superClass().setScale(self, uiScale, uiScale)
	
	local currentVisibility = self:getVisible()
	
	self:setVisible(true, false)
	self.uiScale = uiScale
	local posX, posY = CombineDischargeContext.getBackgroundPosition(uiScale, self:getWidth())
	
	self:setPosition(posX, posY)
	self:storeOriginalPosition()
	self:setVisible(currentVisibility, false)
	self:storeScaledValues()
	self.fadeBackgroundElement:setDimension(1)
	self.fadeBackgroundElement:setPosition(0, 0)
end

function CombineDischargeContext:storeScaledValues()
	self.contextIconOffsetX, self.contextIconOffsetY = self:scalePixelToScreenVector(CombineDischargeContext.POSITION.CONTEXT_ICON)
	self.contextIconSizeX = self:scalePixelToScreenWidth(CombineDischargeContext.SIZE.CONTEXT_ICON[1])
	self.borderOffsetX = self:scalePixelToScreenWidth(CombineDischargeContext.OFFSET.X)
	self.inputIconOffsetX, self.inputIconOffsetX = self:scalePixelToScreenVector(CombineDischargeContext.POSITION.INPUT_ICON)
	self.actionTextOffsetX, self.actionTextOffsetY = self:scalePixelToScreenVector(CombineDischargeContext.POSITION.ACTION_TEXT)
	self.actionTextSize = self:scalePixelToScreenHeight(CombineDischargeContext.TEXT_SIZE.ACTION_TEXT)
	self.targetTextOffsetX, self.targetTextOffsetY = self:scalePixelToScreenVector(CombineDischargeContext.POSITION.TARGET_TEXT)
	self.targetTextSize = self:scalePixelToScreenHeight(CombineDischargeContext.TEXT_SIZE.TARGET_TEXT)
end

function CombineDischargeContext.getBackgroundPosition(scale, width)
	local offX, offY = getNormalizedScreenValues(unpack(CombineDischargeContext.POSITION.BACKGROUND))
	
	return 0.5 - width * 0.5 - offX * scale, g_safeFrameOffsetY - offY * scale
end

function CombineDischargeContext.createBackground()
	local width, height = getNormalizedScreenValues(unpack(CombineDischargeContext.SIZE.BACKGROUND))
	local posX, posY = CombineDischargeContext.getBackgroundPosition(1, width)
	local overlay = Overlay.new(nil, posX, posY, width, height)
	
	return overlay
end

function CombineDischargeContext:createComponents(hudAtlasPath, inputDisplayManager)
	local baseX, baseY = self:getPosition()
	
	self:createFrame(hudAtlasPath, baseX, baseY)
	self:createInputGlyph(hudAtlasPath, baseX, baseY, inputDisplayManager)
	self:createActionIcons(hudAtlasPath, baseX, baseY)
	self:createFadeBackground(hudAtlasPath)
	self:storeOriginalPosition()
end

function CombineDischargeContext:createInputGlyph(hudAtlasPath, baseX, baseY, inputDisplayManager)
	local width, height = getNormalizedScreenValues(unpack(CombineDischargeContext.SIZE.INPUT_ICON))
	local offX, offY = getNormalizedScreenValues(unpack(CombineDischargeContext.POSITION.INPUT_ICON))
	local element = InputGlyphElement.new(inputDisplayManager, width, height)
	local posX = baseX + offX
	local posY = baseY + offY + (self:getHeight() - height) * 0.5
	
	element:setPosition(posX, posY)
	element:setKeyboardGlyphColor(CombineDischargeContext.COLOR.INPUT_ICON)
	self.inputGlyphElement = element
	self:addChild(element)
end

function CombineDischargeContext:createFrame(hudAtlasPath, baseX, baseY)
end

function CombineDischargeContext:createActionIcons(hudAtlasPath, baseX, baseY)
	local posX, posY = getNormalizedScreenValues(unpack(CombineDischargeContext.POSITION.CONTEXT_ICON))
	local width, height = getNormalizedScreenValues(unpack(CombineDischargeContext.SIZE.CONTEXT_ICON))
	local centerY = baseY + (self:getHeight() - height) * 0.5 + posY
	
	for _, iconName in pairs(CombineDischargeContext.CONTEXT_ICON) do
		local iconOverlay = Overlay.new(hudAtlasPath, baseX + posX, centerY, width, height)
		local uvs = CombineDischargeContext.UV[iconName]
		
		iconOverlay:setUVs(GuiUtils.getUVs(uvs))
		iconOverlay:setColor(unpack(CombineDischargeContext.COLOR.CONTEXT_ICON))
		local iconElement = HUDElement.new(iconOverlay)
		
		iconElement:setVisible(false)
		self.contextIconElements[iconName] = iconElement
		self:addChild(iconElement)
	end
end

function CombineDischargeContext:createFadeBackground(hudAtlasPath)
	local _, height = getNormalizedScreenValues(unpack(CombineDischargeContext.SIZE.FADE))
	local overlay = Overlay.new(g_baseUIFilename, 0, 0, 1, height)
	
	overlay:setUVs(g_colorBgUVs)
	overlay:setColor(1, 0, 0, 1)
	setOverlayCornerColor(overlay.overlayId, 0, 0, 0, 0, 0.8)
	setOverlayCornerColor(overlay.overlayId, 1, 0, 0, 0, 0)
	setOverlayCornerColor(overlay.overlayId, 2, 0, 0, 0, 0.8)
	setOverlayCornerColor(overlay.overlayId, 3, 0, 0, 0, 0)
	local element = HUDElement.new(overlay)
	
	element:setVisible(true)
	self:addChild(element)
	self.fadeBackgroundElement = element
end

function CombineDischargeContext:getHidingTranslation()
	local _, height = getNormalizedScreenValues(unpack(CombineDischargeContext.SIZE.FADE))
	
	return 0, -height
end

CombineDischargeContext.UV = {
	[CombineDischargeContext.CONTEXT_ICON.DISCHARGE] = {
		0,
		0,
		48,
		48
	}
}

CombineDischargeContext.SIZE = {
	BACKGROUND = {
		1920,
		60
	},
	INPUT_ICON = {
		36,
		36
	},
	CONTEXT_ICON = {
		64,
		64
	},
	FADE = {
		0,
		180
	}
}

CombineDischargeContext.OFFSET = {
	X = 37.5
}

CombineDischargeContext.TEXT_SIZE = {
	TARGET_TEXT = 26,
	ACTION_TEXT = 14
}

CombineDischargeContext.POSITION = {
	BACKGROUND = {
		0,
		-20
	},
	INPUT_ICON = {
		0,
		3
	},
	CONTEXT_ICON = {
		-6,
		3
	},
	ACTION_TEXT = {
		0,
		-2
	},
	TARGET_TEXT = {
		30,
		0
	}
}

CombineDischargeContext.COLOR = {
	INPUT_ICON = {
		0.0003,
		0.5647,
		0.9822,
		1
	},
	CONTEXT_ICON = {
		0.0003,
		0.5647,
		0.9822,
		1
	},
	ACTION_TEXT = {
		0.0003,
		0.5647,
		0.9822,
		1
	},
	TARGET_TEXT = {
		1,
		1,
		1,
		1
	}
}